!> @file chem_modules.f90
!------------------------------------------------------------------------------!
! This file is part of the PALM model system.
!
! PALM is free software: you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software
! Foundation, either version 3 of the License, or (at your option) any later
! version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 2018-2018 Leibniz Universitaet Hannover
! Copyright 2018-2018 Karlsruhe Institute of Technology
! Copyright 2018-2018 Freie Universitaet Berlin
!------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! 
! 
! Former revisions:
! -----------------
! $Id: chem_modules.f90 3611 2018-12-07 14:14:11Z banzhafs $
! Minor formatting
!
! 3458 2018-10-30 14:51:23Z kanani
! from chemistry branch r3443:
! ??
!
! 3298 2018-10-02 12:21:11Z kanani
! - Minor formatting
! - Introduced Variables for Chemistry emissions Module (Russo)
! - Variables and parameters added for 2d and profile output (basit)
! - Converted scalar  emission factors  to 1-D array of 99 elements each (basit)
! - Removed "__chem" directive (basit)
! 
! 3282 2018-09-27 10:49:12Z basit
! Initial revision
!
! 
!
!
! Authors:
! --------
! @author Farah Kanani-Suehring
! @author Basit Khan
! @author Sabine Banzhaf
! @author Emmanuele Russo
!
!------------------------------------------------------------------------------!
! Description:
! ------------
!> Definition of global palm-4u chemistry variables
!> (Module written to define global palm-4u chemistry variables. basit 16Nov2017)
!------------------------------------------------------------------------------!
!
 MODULE chem_modules

    USE chem_gasphase_mod,                                                     &   
        ONLY: nspec, nvar, spc_names

    USE control_parameters,                                                    &
        ONLY: varnamelength

    USE kinds

    USE statistics,                                                            &
        ONLY: pr_palm


    IMPLICIT NONE

    PUBLIC nspec
    PUBLIC nvar
    PUBLIC spc_names

    INTEGER(iwp), DIMENSION(99) :: cs_pr_index            = 0
    INTEGER(iwp) ::  ibc_cs_b                                                      !< integer flag for bc_cs_b
    INTEGER(iwp) ::  ibc_cs_t                                                      !< integer flag for bc_cs_t
    INTEGER(iwp) ::  cs_pr_count                           = 0 
    INTEGER(iwp) ::  max_pr_cs                             = 0
    INTEGER(iwp) ::  cs_vertical_gradient_level_ind(99,10) = -9999                 !< grid index values of cs_vertical_gradient_level_ind(s)

    LOGICAL      ::  constant_top_csflux(99)               = .TRUE.                !< chem spcs at the top  orig .TRUE.
    LOGICAL      ::  constant_csflux(99)                   = .TRUE.                !< chem spcs at namelist parameter   orig TRUE
    LOGICAL      ::  call_chem_at_all_substeps             = .FALSE.               !< namelist parameter 
    LOGICAL      ::  chem_debug0                           = .FALSE.               !< namelist parameter flag for minimum print output
    LOGICAL      ::  chem_debug1                           = .FALSE.               !< namelist parameter flag for print output
    LOGICAL      ::  chem_debug2                           = .FALSE.               !< namelist parameter flag for further print output
    LOGICAL      ::  chem_gasphase_on                      = .TRUE.                !< namelist parameter 
    LOGICAL      ::  emission_output_required              = .TRUE.                !< Logical Variable for requiring Emission Outputs
    LOGICAL      ::  do_emis                               = .FALSE.               !< Flag for turning on chemistry emissions 
    LOGICAL      ::  cs_pr_namelist_found                  = .FALSE.               !< Namelist parameter: Names of t
    LOGICAL      ::  do_depo                               = .FALSE.               !< namelist parameter for activation of deposition calculation


!
!-- Namelist parameters for creating initial chemistry profiles
    REAL(wp) ::  wall_csflux (99,0:5)               = 0.0_wp                        !< namelist parameter
    REAL(wp) ::  cs_vertical_gradient (99,10)       = 0.0_wp                        !< namelist parameter
    REAL(wp) ::  cs_vertical_gradient_level (99,10) = -999999.9_wp                  !< namelist parameter
    REAL(wp) ::  top_csflux ( 99 )                  = 0.0_wp                        !< namelist parameter 
    REAL(wp) ::  cs_surface_initial_change(99)      = 0.0_wp                        !< namelist parameter
    REAL(wp) ::  surface_csflux(99 )                = 0.0_wp                        !< namelist parameter: fluxes where 'surface_csflux_name' is in the namelist

    REAL(wp), DIMENSION(:), ALLOCATABLE               ::  bc_cs_t_val
    REAL(wp), DIMENSION(:), ALLOCATABLE               ::  css                       !< scaling parameter for chem spcs
    REAL(wp), DIMENSION(99)                           ::  cs_surface = 0.0_wp       !< Namelist parameter: Surface conc of chem spcs'
    REAL(wp), DIMENSION(99,100)                       ::  cs_heights = 9999999.9_wp !< Namelist parameter: Height lvls(m) for cs_profiles
    REAL(wp), DIMENSION(99,100)                       ::  cs_profile = 9999999.9_wp !< Namelist parameter: Chem conc for each spcs defined


#if defined( __nopointer )
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE, TARGET   ::  cs                        !< chem spcs
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE, TARGET   ::  cs_p                      !< prognostic value of chem spc
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE, TARGET   ::  tcs_m                     !< weighted tendency of cs for previous sub-timestep (Runge-Kutta)

#else                                                               
!
!-- Use pointers cs, cs_p and tcs_m to point arrays cs_1, cs_2, and cs_3
    REAL(wp), DIMENSION(:,:,:,:), ALLOCATABLE, TARGET ::  cs_1                      !< pointer for swapping of timelevels for respective quantity
    REAL(wp), DIMENSION(:,:,:,:), ALLOCATABLE, TARGET ::  cs_2                      !< pointer for swapping of timelevels for respective quantity
    REAL(wp), DIMENSION(:,:,:,:), ALLOCATABLE, TARGET ::  cs_3                      !< pointer for swapping of timelevels for respective quantity
    REAL(wp), DIMENSION(:,:,:), POINTER               ::  cs                        !< pointer: sgs chem spcs)
    REAL(wp), DIMENSION(:,:,:), POINTER               ::  cs_p                      !< pointer: prognostic value of sgs chem spcs 
    REAL(wp), DIMENSION(:,:,:), POINTER               ::  tcs_m                     !< pointer:

#endif                                                                            
 
    CHARACTER (LEN=20)                ::  bc_cs_b             = 'dirichlet'         !< namelist parameter
    CHARACTER (LEN=20)                ::  bc_cs_t             = 'initial_gradient'  !< namelist parameter
    CHARACTER (LEN=11), DIMENSION(99) ::  cs_name             = 'novalue'           !< Namelist parameter: chem spcs names 
    CHARACTER (LEN=11), DIMENSION(99) ::  cs_profile_name     = 'novalue'           !< Namelist parameter: Names of the chem for profiles
    CHARACTER (LEN=11), DIMENSION(99) ::  surface_csflux_name = 'novalue'           !< Namelist parameter: chem species surface fluxes names
                                                                                    !< active chem spcs, default is 'novalue')  ????
    CHARACTER (LEN=80)                ::  mode_emis           ='PARAMETERIZED'      !< Mode of chemistry emissions: DEFAULT .OR. EXPERT .OR. 
                                                                                    !< PARAMETERIZED
    CHARACTER (LEN=80)                ::  time_fac_type       ='MDH'                !< Type of time treatment in the emis DEFAULT mode: HOUR .OR. MDH 
    CHARACTER (LEN=80)                ::  daytype_mdh         ='workday'            !< Type of day in the MDH case: workday, weekend, holiday
    CHARACTER (LEN=11), DIMENSION(99) ::  data_output_pr_cs   = 'novalue'           !< Namelist parameter: Names of the che    m for profile output
                                                                                    !< by cs_name for each height lvls defined by cs_heights
!
!-- Namelist parameters for chem_emissions
    INTEGER(iwp) ::  main_street_id = 0
    INTEGER(iwp) ::  max_street_id  = 0
    INTEGER(iwp) ::  side_street_id = 0
!
!-- Constant emission factors
    REAL(wp) ::  emiss_factor_main ( 99 ) = -9999.0_wp
    REAL(wp) ::  emiss_factor_side ( 99 ) = -9999.0_wp
!    
!-- Other Emissions Variables
    INTEGER(iwp) ::  nspec_out                                                     !< Output of routine chem_emis_matching with 
                                                                                   !< number of matched species
    REAL(wp),ALLOCATABLE, DIMENSION(:,:,:,:) ::  emis_distribution                 !> Emissions Final Values (main module output)

    INTEGER(iwp),ALLOCATABLE,DIMENSION(:)    ::  match_spec_input                  !< Index of Input chem species for matching routine 
    INTEGER(iwp),ALLOCATABLE,DIMENSION(:)    ::  match_spec_model                  !< Index of Model chem species for matching routine
    INTEGER(iwp),ALLOCATABLE,DIMENSION(:)    ::  match_spec_voc_input              !< index of VOC input components matching the model's VOCs
    INTEGER(iwp),ALLOCATABLE,DIMENSION(:)    ::  match_spec_voc_model              !< index of VOC model species matching the input VOCs comp.
    INTEGER(iwp),DIMENSION(:)                ::  match_spec_pm(1:3)                !< results of matching the input and model's PMs
    INTEGER(iwp),DIMENSION(:)                ::  match_spec_nox(1:2)               !< results of matching the input and model's NOx
    INTEGER(iwp),DIMENSION(:)                ::  match_spec_sox(1:2)               !< results of matching the input and model's SOx! 
                                                                                  

!
!-- Selected atomic/molecular weights:
    REAL, PARAMETER        ::  xm_H     =    1.00790e-3           !< kg/mol
    REAL, PARAMETER        ::  xm_N     =   14.00670e-3           !< kg/mol
    REAL, PARAMETER        ::  xm_C     =   12.01115e-3           !< kg/mol
    REAL, PARAMETER        ::  xm_S     =   32.06400e-3           !< kg/mol
    REAL, PARAMETER        ::  xm_O     =   15.99940e-3           !< kg/mol
    REAL, PARAMETER        ::  xm_F     =   18.99840e-3           !< kg/mol
    REAL, PARAMETER        ::  xm_Na    =   22.98977e-3           !< kg/mol
    REAL, PARAMETER        ::  xm_Cl    =   35.45300e-3           !< kg/mol
    REAL, PARAMETER        ::  xm_Rn222 =  222.00000e-3           !< kg/mol
    REAL, PARAMETER        ::  xm_Pb210 =  210.00000e-3           !< kg/mol
    REAL, PARAMETER        ::  xm_Ca    =   40.07800e-3           !< kg/mol
    REAL, PARAMETER        ::  xm_K     =   39.09800e-3           !< kg/mol
    REAL, PARAMETER        ::  xm_Mg    =   24.30500e-3           !< kg/mol
    REAL, PARAMETER        ::  xm_Pb    =  207.20000e-3           !< kg/mol
    REAL, PARAMETER        ::  xm_Cd    =  112.41000e-3           !< kg/mol
    
    REAL, PARAMETER        ::  xm_h2o   = xm_H * 2 + xm_O         !< kg/mol
    REAL, PARAMETER        ::  xm_o3    = xm_O * 3                !< kg/mol
    REAL, PARAMETER        ::  xm_N2O5  = xm_N * 2 + xm_O * 5     !< kg/mol
    REAL, PARAMETER        ::  xm_HNO3  = xm_H + xm_N + xm_O * 3  !< kg/mol
    REAL, PARAMETER        ::  xm_NH4   = xm_N + xm_H * 4         !< kg/mol
    REAL, PARAMETER        ::  xm_SO4   = xm_S + xm_O * 4         !< kg/mol
    REAL, PARAMETER        ::  xm_NO3   = xm_N + xm_O * 3         !< kg/mol
    REAL, PARAMETER        ::  xm_CO2   = xm_C + xm_O * 2         !< kg/mol
    
!
!-- mass of air
    REAL, PARAMETER        ::  xm_air   =  28.964e-3              !< kg/mol
        
!
!-- dummy weight, used for complex molecules:
    REAL, PARAMETER        ::  xm_dummy =  1000.0e-3              ! kg/mol

    
    SAVE
 END MODULE chem_modules

